package fr.uha.gasser.pathfinding.algorithm;

import fr.uha.gasser.pathfinding.model.Grid;

import java.awt.*;
import java.util.*;
import java.util.List;

class AStarSearch {

    private final Node start;
    private final Node goal;
    private final Grid grid;

    AStarSearch(Node start, Node goal, Grid grid) {
        this.start = start;
        this.goal = goal;
        this.grid = grid;
    }


    private Node search() {
        PriorityQueue<Node> frontier = new PriorityQueue<>();
        Set<Node> explored = new HashSet<>();
        frontier.add(start);

        while (!frontier.isEmpty()) {
            Node currentNode = frontier.poll();
            explored.add(currentNode);

            if (currentNode.getLoc().equals(goal.getLoc())) {
                return currentNode;
            }

            for (Node neighbor : getNeighbors(currentNode)) {
                // Ignore the neighbor which is already evaluated.
                if (explored.contains(neighbor)) continue;
                if (!frontier.contains(neighbor)) {
                    neighbor.setParent(currentNode);
                    neighbor.cost++;
                    neighbor.hValue = neighbor.cost + neighbor.loc.distance(goal.loc);
                    frontier.add(neighbor);
                }
            }
        }

        return null;
    }

    List<Node> shortestPath() {
        Node currentNode = search();
        LinkedList<Node> path = new LinkedList<>();
        while (currentNode.getParent() != null) {
            path.addFirst(currentNode);
            currentNode = currentNode.getParent();
        }

        return path;
    }

    private List<Node> getNeighbors(Node current) {
        LinkedList<Node> neighbors = new LinkedList<>();
        for (Direction direction : Direction.values()) {
            final Node neighbor = getNeighbor(current, direction);
            // If neighbor is a wall or off the grid (null), don't add it to the list
            if (neighbor != null) neighbors.add(neighbor);
        }

        return neighbors;
    }

    private Node getNeighbor(Node current, Direction direction) {
        final Dimension gridSize = grid.getSize();
        final Node node = new Node(current);
        final Point loc = new Point(current.loc.x + direction.x, current.loc.y + direction.y);
        if (grid.get(loc).isWall()) return null;
        // Don't overflow off the grid
        if (loc.x > gridSize.width || loc.y > gridSize.height) return null;
        node.setLoc(loc);
        return node;
    }

    private enum Direction {
        UP(0, 1),
        DOWN(0, -1),
        LEFT(-1, 0),
        LEFT_UP(-1, 1),
        LEFT_DOWN(-1, -1),
        RIGHT(1, 0),
        RIGHT_UP(1, 1),
        RIGHT_DOWN(1, -1);

        public final int x;
        public final int y;

        Direction(int x, int y) {
            this.x = x;
            this.y = y;
        }
    }

}