package fr.uha.gasser.pathfinding.algorithm;

import fr.uha.gasser.pathfinding.model.Grid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.*;
import java.awt.*;
import java.util.Collections;
import java.util.List;

public class AStarWorker extends SwingWorker<List<Node>, List<Node>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(AStarWorker.class);

    private final Node start;
    private final Node goal;
    private final Grid target;
    private List<Node> path;

    public AStarWorker(Node start, Node goal, Grid target) {
        this.start = start;
        this.goal = goal;
        this.target = target;
    }

    @Override
    protected List<Node> doInBackground() throws Exception {
        AStarSearch aStar = new AStarSearch(start, goal, target);
        path = aStar.shortestPath();
        return path;
    }

    @Override
    protected void done() {
        if (this.isCancelled()) return;
        if (path.equals(Collections.EMPTY_LIST)) {
            LOGGER.warn("There is no path between the nodes {} and {}.", start, goal);
            return;
        }
        path.forEach(node -> {
            if (! node.loc.equals(goal.loc)) target.setColor(node.loc, Color.BLUE);
        });
    }
}
