package fr.uha.gasser.pathfinding.model;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class Grid implements Iterable<Cell> {

    private final static Border lineBorder = BorderFactory.createLineBorder(Color.BLACK, 1);
    private static final Point INVALID = new Point(-256, -256);

    private final List<Cell> cells;
    private final Dimension size;
    private final int cellSize;

    public Grid(int rows, int cols, int cellSize) {
        this.size = new Dimension(rows, cols);
        this.cellSize = cellSize;
        this.cells = new ArrayList<>(rows*cols);
        initGrid(rows*cols);
    }

    private void initGrid(int size) {
        for (int i = 0; i < size; i++) {
            final JLabel label = new JLabel();
            // Force cell size
            label.setPreferredSize(new Dimension(cellSize, cellSize));
            label.setMinimumSize(label.getPreferredSize());
            label.setMaximumSize(label.getPreferredSize());
            label.setBorder(lineBorder);
            label.setOpaque(true);
            final Point pos = new Point(i % this.size.width, i / this.size.height);
            cells.add(new Cell(pos, label));
        }
    }

    public Cell get(Point loc) {
        return cells.get(loc.x + size.width * loc.y);
    }

    public Cell get(JLabel content) {
        for (Cell c : cells) {
            if (c.getContent() == content) return c;
        }
        // Not found
        return new Cell(INVALID, null);
    }

    public final Dimension getSize() {
        return size;
    }

    public void setColor(Point loc, Color color) {
        final Cell c = cells.get(loc.x + size.width * loc.y);
        final JLabel content = c.getContent();
        content.setBackground(color);
        content.revalidate();
    }

    public void setWall(Point loc, boolean wall) {
        final Cell c = cells.get(loc.x + size.width * loc.y);
        c.setWall(wall);
        c.getContent().setBackground(wall ? Color.BLACK : null);
        c.getContent().revalidate();
    }

    public void clear() {
        for (Cell cell : cells) {
            cell.getContent().setBackground(null);
            cell.setWall(false);
        }
    }

    @Override
    public Iterator<Cell> iterator() {
        return cells.iterator();
    }

}
